from datetime import date, timedelta
import datetime
import requests
from pathlib import Path
import click
from tqdm import tqdm
from data import data_dir, connect
from lxml import etree
import pandas as pd

@click.group()
def cli():
    ...

@cli.command()
@click.option('--directory', type=Path, default=data_dir())
@click.option('--database', type=Path, default=data_dir() / "stories.duckdb")
def load(directory, database):
    stories = directory / "stories.csv"
    related = directory / "related.csv"
    db = connect()

    db.sql(f"""
        CREATE TABLE stories AS
        SELECT 
            * 
        FROM read_csv_auto('{stories}')
    """)

    db.sql(f"""
        CREATE TABLE related_stories AS
        SELECT 
            * 
        FROM read_csv_auto('{related}')
    """)
    db.close()

@cli.command()
@click.option('-o', 'output_dir', type=Path, default=data_dir() / "memeorandum")
def download(output_dir):
    day = timedelta(days=1)
    cur = date(2005, 10, 1)
    end = date.today()
    dates = []
    while cur <= end:
        if not (output_dir / f"{cur.strftime('%y-%m-%d')}.html").exists():
            dates.append(cur)
        cur = cur + day
    date_iter = tqdm(dates, postfix="test")
    for i in date_iter:
        date_iter.set_postfix_str(f"{i}")
        save_as = output_dir / f"{i.strftime('%y-%m-%d')}.html"
        if save_as.exists():
            continue
        url = f"https://www.memeorandum.com/{i.strftime('%y%m%d')}/h2000"
        r = requests.get(url)
        with open(save_as, 'w') as f:
            f.write(r.text)


@cli.command()
@click.option('-d', '--directory', type=Path, default=data_dir() / "memeorandum", show_default=True)
@click.option('-o', '--output_dir', type=Path, default=data_dir(), show_default=True)
def parse(directory, output_dir):
    """parse the html files on disk into a structured csv format."""
    directory = data_dir() / "memeorandum"
    parser = etree.HTMLParser()
    pages = [f for f in directory.glob("*.html")]
    published = []
    others = []
    # page = pages[0]
    page_iter = tqdm(pages, postfix="starting")
    for page in page_iter:
        page_iter.set_postfix_str(f"{page}")
        date = datetime.datetime.strptime(page.stem, '%y-%m-%d')
        tree = etree.parse(str(page), parser)
        root = tree.getroot()
        if root is None:
            print(f"error opening {page}")
            continue
        items = root.xpath("//div[contains(@class, 'item')]")

        # item = items[0]
        for item in items:
            out = dict()
            citation = item.xpath('./cite')
            if not citation:
                continue
            author = citation[0]
            if author.text: 
                author = ''.join(author.text.split('/')[:-1]).strip()
            else:
                author = ''
            out['author'] = author
            try:
                publisher_url = citation[0].getchildren()[0].get('href')
                publisher = citation[0].getchildren()[0].text
            except IndexError as e:
                print(f"error with citation url: {page}")
            out['publisher'] = publisher
            out['publisher_url'] = publisher_url

            title = item.xpath('.//strong/a')[0].text
            out['title'] = title

            url = item.xpath('.//strong/a')[0].get('href')
            out['url'] = url

            item_id = hash((page.stem, url))
            out['id'] = item_id

            old_id = hash((title, page.stem, publisher_url))
            out['old_id'] = old_id
            published.append(out)

            related = item.xpath(".//span[contains(@class, 'mls')]/a")
            # relation = related[0]
            for relation in related:
                another = dict()
                another['url'] = relation.get('href')
                another['publisher'] = relation.text
                another['parent_id'] = item_id
                others.append(another)
    df = pd.DataFrame(published)
    df.to_csv(output_dir / 'stories_v2.csv', sep='|', index=False)
    df = pd.DataFrame(others)
    df.to_csv(output_dir / 'related_v2.csv', sep='|', index=False)

@cli.command()
def normalize():
    DB = connect()
    DB.sql("""
        create table publishers as
        select
            row_number() over(order by publisher) as id
            ,publisher
            ,publisher_url
        from stories
        group by publisher, publisher_url
    """)


if __name__ == "__main__":
    cli()
